searchData={"items":[{"type":"module","title":"Delta","doc":"","ref":"Delta.html"},{"type":"function","title":"Delta.compact/1","doc":"Compacts Delta to satisfy [compactness](https://quilljs.com/guides/designing-the-delta-format/#compact) requirement.","ref":"Delta.html#compact/1"},{"type":"function","title":"Examples - Delta.compact/1","doc":"iex> delta = [Op.insert(\"Hel\"), Op.insert(\"lo\"), Op.insert(\"World\", %{\"bold\" => true})]\n    iex> Delta.compact(delta)\n    [%{\"insert\" => \"Hello\"}, %{\"insert\" => \"World\", \"attributes\" => %{\"bold\" => true}}]","ref":"Delta.html#compact/1-examples"},{"type":"function","title":"Delta.compose/2","doc":"Returns a new Delta that is equivalent to applying the operations of one Delta, followed by another Delta","ref":"Delta.html#compose/2"},{"type":"function","title":"Examples - Delta.compose/2","doc":"iex> a = [Op.insert(\"abc\")]\n    iex> b = [Op.retain(1), Op.delete(1)]\n    iex> Delta.compose(a, b)\n    [%{\"insert\" => \"ac\"}]","ref":"Delta.html#compose/2-examples"},{"type":"function","title":"Delta.compose_all/1","doc":"Returns a new Delta that is equivalent to applying Deltas one by one","ref":"Delta.html#compose_all/1"},{"type":"function","title":"Examples - Delta.compose_all/1","doc":"iex> a = [Op.insert(\"ac\")]\n    iex> b = [Op.retain(1), Op.insert(\"b\")]\n    iex> c = [Op.delete(1)]\n    iex> Delta.compose_all([a, b, c])\n    [%{\"insert\" => \"bc\"}]","ref":"Delta.html#compose_all/1-examples"},{"type":"function","title":"Delta.concat/2","doc":"Concatenates two Deltas.","ref":"Delta.html#concat/2"},{"type":"function","title":"Examples - Delta.concat/2","doc":"iex> a = [Op.insert(\"Hel\")]\n    iex> b = [Op.insert(\"lo\")]\n    iex> Delta.concat(a, b)\n    [%{\"insert\" => \"Hello\"}]","ref":"Delta.html#concat/2-examples"},{"type":"function","title":"Delta.diff/2","doc":"Returns a delta representing the difference between two documents.","ref":"Delta.html#diff/2"},{"type":"function","title":"Examples - Delta.diff/2","doc":"iex> a = [Op.insert(\"Hello\")]\n    iex> b = [Op.insert(\"Hello!\")]\n    iex> diff = Delta.diff(a, b)\n    [\n      %{\"retain\" => 5},\n      %{\"insert\" => \"!\"}\n    ]\n    iex> Delta.compose(a, diff) == b\n    true","ref":"Delta.html#diff/2-examples"},{"type":"function","title":"Delta.get_handler/1","doc":"","ref":"Delta.html#get_handler/1"},{"type":"function","title":"Delta.get_handler!/1","doc":"","ref":"Delta.html#get_handler!/1"},{"type":"function","title":"Delta.invert/2","doc":"Returns an inverted delta that has the opposite effect of against a base document delta.\n\nThat is base |> Delta.compose(change) |> Delta.compose(inverted) == base.","ref":"Delta.html#invert/2"},{"type":"function","title":"Examples - Delta.invert/2","doc":"iex> base = [Op.insert(\"Hello\\nWorld\")]\n    iex> change = [\n    ...>   Op.retain(6, %{\"bold\" => true}),\n    ...>   Op.delete(5),\n    ...>   Op.insert(\"!\"),\n    ...> ]\n    iex> inverted = Delta.invert(change, base)\n    [\n      %{\"retain\" => 6, \"attributes\" => %{\"bold\" => nil}},\n      %{\"insert\" => \"World\"},\n      %{\"delete\" => 1},\n    ]\n    iex> base |> Delta.compose(change) |> Delta.compose(inverted) == base\n    true","ref":"Delta.html#invert/2-examples"},{"type":"function","title":"Delta.push/2","doc":"Pushes an operation to a reversed Delta honouring semantics.\n\nNote: that reversed delta does not represent a reversed text, but rather a list\nof operations that was naively reversed during programmatic manipulations.\nThis function is normally only used by other functions which reverse the list\nback in the end.","ref":"Delta.html#push/2"},{"type":"function","title":"Examples - Delta.push/2","doc":"iex> delta = [Op.insert(\"World\", %{\"italic\" => true}), Op.insert(\"Hello\", %{\"bold\" => true})]\n    iex> op = Op.insert(\"!\")\n    iex> Delta.push(delta, op)\n    [%{\"insert\" => \"!\"}, %{\"insert\" => \"World\", \"attributes\" => %{\"italic\" => true}}, %{\"insert\" => \"Hello\", \"attributes\" => %{\"bold\" => true}}]\n\n    iex> delta = [Op.insert(\"World\"), Op.insert(\"Hello\", %{\"bold\" => true})]\n    iex> op = Op.insert(\"!\")\n    iex> Delta.push(delta, op)\n    [%{\"insert\" => \"World!\"}, %{\"insert\" => \"Hello\", \"attributes\" => %{\"bold\" => true}}]","ref":"Delta.html#push/2-examples"},{"type":"function","title":"Delta.size/1","doc":"Returns the size of delta.","ref":"Delta.html#size/1"},{"type":"function","title":"Examples - Delta.size/1","doc":"iex> delta = [Op.insert(\"abc\"), Op.retain(2), Op.delete(1)]\n    iex> Delta.size(delta)\n    6","ref":"Delta.html#size/1-examples"},{"type":"function","title":"Delta.slice/3","doc":"Attempts to take `len` characters starting from `index`.\n\nNote: note that due to the way it's implemented this operation can potentially\nraise if the resulting text isn't a valid UTF-8 encoded string","ref":"Delta.html#slice/3"},{"type":"function","title":"Examples - Delta.slice/3","doc":"iex> delta = [Op.insert(\"Hello World\")]\n    iex> Delta.slice(delta, 6, 3)\n    [%{\"insert\" => \"Wor\"}]\n\n    iex> delta = [Op.insert(\"01🙋45\")]\n    iex> Delta.slice(delta, 1, 2)\n    ** (RuntimeError) Encoding failed in take_partial {\"1🙋45\", %{\"insert\" => \"1🙋45\"}, 2, {:incomplete, \"1\", <<216, 61>>}, {:error, \"\", <<222, 75, 0, 52, 0, 53>>}}","ref":"Delta.html#slice/3-examples"},{"type":"function","title":"Delta.slice_max/3","doc":"Takes `len` or fewer characters from `index` position. Variable `len` allows\nto not cut things like emojis in half.","ref":"Delta.html#slice_max/3"},{"type":"function","title":"Examples - Delta.slice_max/3","doc":"iex> delta = [Op.insert(\"Hello World\")]\n    iex> Delta.slice_max(delta, 6, 3)\n    [%{\"insert\" => \"Wor\"}]\n\n    iex> delta = [Op.insert(\"01🙋45\")]\n    iex> Delta.slice_max(delta, 1, 2)\n    [%{\"insert\" => \"1\"}]","ref":"Delta.html#slice_max/3-examples"},{"type":"function","title":"Delta.split/3","doc":"Splits delta at the given index.","ref":"Delta.html#split/3"},{"type":"function","title":"Options - Delta.split/3","doc":"* `:align` - when `true`, allow moving index left if\n    we're likely to split a grapheme otherwise.","ref":"Delta.html#split/3-options"},{"type":"function","title":"Examples - Delta.split/3","doc":"iex> delta = [Op.insert(\"Hello World\")]\n    iex> Delta.split(delta, 5)\n    {[%{\"insert\" => \"Hello\"}], [%{\"insert\" => \" World\"}]}\n\n    iex> delta = [Op.insert(\"01🙋45\")]\n    iex> Delta.split(delta, 3, align: true)\n    {[%{\"insert\" => \"01\"}], [%{\"insert\" => \"🙋45\"}]}\n\n    iex> delta = [Op.insert(\"a\"), Op.insert(\"b\", %{\"bold\" => true})]\n    iex> Delta.split(delta, 3)\n    {[%{\"insert\" => \"a\"}, %{\"insert\" => \"b\", \"attributes\" => %{\"bold\" => true}}], []}","ref":"Delta.html#split/3-examples"},{"type":"function","title":"Delta.transform/3","doc":"Transforms given delta against another's operations.\n\nThis accepts an optional priority argument (default: false), used to break ties.\nIf true, the first delta takes priority over other, that is, its actions are considered to happen \"first.\"","ref":"Delta.html#transform/3"},{"type":"function","title":"Examples - Delta.transform/3","doc":"iex> a = [Op.insert(\"a\")]\n    iex> b = [Op.insert(\"b\"), Op.retain(5), Op.insert(\"c\")]\n    iex> Delta.transform(a, b, true)\n    [\n      %{\"retain\" => 1},\n      %{\"insert\" => \"b\"},\n      %{\"retain\" => 5},\n      %{\"insert\" => \"c\"},\n    ]\n    iex> Delta.transform(a, b)\n    [\n      %{\"insert\" => \"b\"},\n      %{\"retain\" => 6},\n      %{\"insert\" => \"c\"},\n    ]","ref":"Delta.html#transform/3-examples"},{"type":"type","title":"Delta.t/0","doc":"","ref":"Delta.html#t:t/0"},{"type":"module","title":"Delta.Attr","doc":"","ref":"Delta.Attr.html"},{"type":"function","title":"Delta.Attr.compose/3","doc":"","ref":"Delta.Attr.html#compose/3"},{"type":"function","title":"Delta.Attr.diff/2","doc":"","ref":"Delta.Attr.html#diff/2"},{"type":"function","title":"Delta.Attr.invert/2","doc":"","ref":"Delta.Attr.html#invert/2"},{"type":"function","title":"Delta.Attr.transform/3","doc":"","ref":"Delta.Attr.html#transform/3"},{"type":"behaviour","title":"Delta.EmbedHandler","doc":"A module implementing `EmbedHandler` behaviour is required to make `compose`, `transform` and `invert` operations possible for custom embeds.\n\nSuppose we want to have a custom `image` embed handler, that will always prefer image urls that start with `https://`\n\n    defmodule ImageEmbed do\n      @behaviour Delta.EmbedHandler\n\n      @impl Delta.EmbedHandler\n      def name, do: \"image\"\n\n      @impl Delta.EmbedHandler\n      def compose(url1, url2, _keep_nil?) do\n        choose_https(url1, url2)\n      end\n\n      @impl Delta.EmbedHandler\n      def transform(url1, url2, _priority? \\ false) do\n        choose_https(url1, url2)\n      end\n\n      @impl Delta.EmbedHandler\n      def invert(change, base) do\n        choose_https(change, base)\n      end\n\n      defp choose_https(url1, url2) do\n        case {url1, url2} do\n          {\"https\" <> _ , _} -> url1\n\n          {_, \"https\" <> _} -> url2\n\n          {_, _} -> url1\n        end\n      end\n    end\n\nNow we just need to add our module to config:\n\n    config :delta, custom_embeds: [ImageEmbed]","ref":"Delta.EmbedHandler.html"},{"type":"behaviour","title":"Examples - Delta.EmbedHandler","doc":"iex> base = Op.insert(%{\"image\" => \"update me\"})\n    iex> a = Op.retain(%{\"image\" => \"http://quilljs.com/assets/images/icon.png\"})\n    iex> b = Op.retain(%{\"image\" => \"https://quilljs.com/assets/images/icon.png\"})\n    iex> Delta.transform([base, a], [base, b])\n    [\n      %{\"insert\" => %{\"image\" => \"update me\"}},\n      %{\"retain\" => 1},\n      %{\"retain\" => %{\"image\" => \"https://quilljs.com/assets/images/icon.png\"}}\n    ]","ref":"Delta.EmbedHandler.html#module-examples"},{"type":"callback","title":"Delta.EmbedHandler.compose/3","doc":"","ref":"Delta.EmbedHandler.html#c:compose/3"},{"type":"callback","title":"Delta.EmbedHandler.diff/2","doc":"","ref":"Delta.EmbedHandler.html#c:diff/2"},{"type":"callback","title":"Delta.EmbedHandler.invert/2","doc":"","ref":"Delta.EmbedHandler.html#c:invert/2"},{"type":"callback","title":"Delta.EmbedHandler.name/0","doc":"","ref":"Delta.EmbedHandler.html#c:name/0"},{"type":"callback","title":"Delta.EmbedHandler.transform/3","doc":"","ref":"Delta.EmbedHandler.html#c:transform/3"},{"type":"type","title":"Delta.EmbedHandler.embed/0","doc":"","ref":"Delta.EmbedHandler.html#t:embed/0"},{"type":"type","title":"Delta.EmbedHandler.t/0","doc":"","ref":"Delta.EmbedHandler.html#t:t/0"},{"type":"module","title":"Delta.Op","doc":"","ref":"Delta.Op.html"},{"type":"function","title":"Delta.Op.compose/2","doc":"","ref":"Delta.Op.html#compose/2"},{"type":"function","title":"Delta.Op.delete/2","doc":"A shorthand for `new(\"delete\", value, attributes)`. See `new/3`.","ref":"Delta.Op.html#delete/2"},{"type":"function","title":"Examples - Delta.Op.delete/2","doc":"iex> Op.delete(1, %{\"bold\" => true})\n    %{\"delete\" => 1, \"attributes\" => %{\"bold\" => true}}","ref":"Delta.Op.html#delete/2-examples"},{"type":"function","title":"Delta.Op.delete?/2","doc":"A shorthand for `type?(op, \"delete\", type)`. See `type?/3`.","ref":"Delta.Op.html#delete?/2"},{"type":"function","title":"Examples - Delta.Op.delete?/2","doc":"iex> Op.delete(1) |> Op.delete?()\n    true","ref":"Delta.Op.html#delete?/2-examples"},{"type":"function","title":"Delta.Op.get_embed_data!/2","doc":"Gets two embeds' data. An embed is always a [one-key map](https://quilljs.com/docs/delta/#embeds)","ref":"Delta.Op.html#get_embed_data!/2"},{"type":"function","title":"Examples - Delta.Op.get_embed_data!/2","doc":"iex> Op.get_embed_data!(\n    ...>   %{\"image\" => \"https://quilljs.com/assets/images/icon.png\"},\n    ...>   %{\"image\" => \"https://quilljs.com/assets/images/icon2.png\"}\n    ...> )\n    {\"image\", \"https://quilljs.com/assets/images/icon.png\", \"https://quilljs.com/assets/images/icon2.png\"}","ref":"Delta.Op.html#get_embed_data!/2-examples"},{"type":"function","title":"Delta.Op.has_attributes?/1","doc":"Returns true if operation has attributes","ref":"Delta.Op.html#has_attributes?/1"},{"type":"function","title":"Examples - Delta.Op.has_attributes?/1","doc":"iex> Op.has_attributes?(%{\"insert\" => \"Hello\", \"attributes\" => %{\"bool\" => true}})\n    true\n\n    iex> Op.has_attributes?(%{\"insert\" => \"Hello\"})\n    false","ref":"Delta.Op.html#has_attributes?/1-examples"},{"type":"function","title":"Delta.Op.insert/2","doc":"A shorthand for `new(\"insert\", value, attributes)`. See `new/3`.","ref":"Delta.Op.html#insert/2"},{"type":"function","title":"Examples - Delta.Op.insert/2","doc":"iex> Op.insert(\"Hello\", %{\"bold\" => true})\n    %{\"insert\" => \"Hello\", \"attributes\" => %{\"bold\" => true}}","ref":"Delta.Op.html#insert/2-examples"},{"type":"function","title":"Delta.Op.insert?/2","doc":"A shorthand for `type?(op, \"insert\", type)`. See `type?/3`.","ref":"Delta.Op.html#insert?/2"},{"type":"function","title":"Examples - Delta.Op.insert?/2","doc":"iex> Op.insert(\"Hello\") |> Op.insert?()\n    true","ref":"Delta.Op.html#insert?/2-examples"},{"type":"function","title":"Delta.Op.new/3","doc":"Create a new operation.\n\nNote that operations _are_ maps, and not structs.","ref":"Delta.Op.html#new/3"},{"type":"function","title":"Examples - Delta.Op.new/3","doc":"iex> Op.new(\"insert\", \"Hello\", %{\"bold\" => true})\n    %{\"insert\" => \"Hello\", \"attributes\" => %{\"bold\" => true}}","ref":"Delta.Op.html#new/3-examples"},{"type":"function","title":"Delta.Op.retain/2","doc":"A shorthand for `new(\"retain\", value, attributes)`. See `new/3`.","ref":"Delta.Op.html#retain/2"},{"type":"function","title":"Examples - Delta.Op.retain/2","doc":"iex> Op.retain(1, %{\"bold\" => true})\n    %{\"retain\" => 1, \"attributes\" => %{\"bold\" => true}}","ref":"Delta.Op.html#retain/2-examples"},{"type":"function","title":"Delta.Op.retain?/2","doc":"A shorthand for `type?(op, \"insert\", type)`. See `type?/3`.","ref":"Delta.Op.html#retain?/2"},{"type":"function","title":"Examples - Delta.Op.retain?/2","doc":"iex> Op.retain(1) |> Op.retain?()\n    true","ref":"Delta.Op.html#retain?/2-examples"},{"type":"function","title":"Delta.Op.size/1","doc":"Returns operation size.","ref":"Delta.Op.html#size/1"},{"type":"function","title":"Examples - Delta.Op.size/1","doc":"iex> Op.insert(\"Hello\") |> Op.size()\n    5\n\n    iex> Op.retain(3) |> Op.size()\n    3","ref":"Delta.Op.html#size/1-examples"},{"type":"function","title":"Delta.Op.take/3","doc":"Takes `length` characters from an operation and returns it together with the\nremaining part in a tuple.","ref":"Delta.Op.html#take/3"},{"type":"function","title":"Options - Delta.Op.take/3","doc":"* `:align` - when `true`, allow moving index left if\n    we're likely to split a grapheme otherwise.","ref":"Delta.Op.html#take/3-options"},{"type":"function","title":"Examples - Delta.Op.take/3","doc":"iex> Op.insert(\"Hello\") |> Op.take(3)\n    {%{\"insert\" => \"Hel\"}, %{\"insert\" => \"lo\"}}\n\n    iex> assert_raise RuntimeError, fn -> Op.insert(\"🏴󠁧󠁢󠁳󠁣󠁴󠁿\") |> Op.take(1) end\n\n    iex> Op.insert(\"🏴󠁧󠁢󠁳󠁣󠁴󠁿\") |> Op.take(1, align: true)\n    {%{\"insert\" => \"\"}, %{\"insert\" => \"🏴󠁧󠁢󠁳󠁣󠁴󠁿\"}}","ref":"Delta.Op.html#take/3-examples"},{"type":"function","title":"Delta.Op.text_size/1","doc":"Returns text size.","ref":"Delta.Op.html#text_size/1"},{"type":"function","title":"Examples - Delta.Op.text_size/1","doc":"iex> Op.text_size(\"Hello\")\n    5\n\n    iex> Op.text_size(\"🏴󠁧󠁢󠁳󠁣󠁴󠁿\")\n    14","ref":"Delta.Op.html#text_size/1-examples"},{"type":"function","title":"Delta.Op.transform/3","doc":"","ref":"Delta.Op.html#transform/3"},{"type":"function","title":"Delta.Op.transform/4","doc":"","ref":"Delta.Op.html#transform/4"},{"type":"function","title":"Delta.Op.type?/3","doc":"Returns true if operation is of type `type`. Optionally check against more specific `value_type`.","ref":"Delta.Op.html#type?/3"},{"type":"function","title":"Examples - Delta.Op.type?/3","doc":"iex> Op.insert(\"Hello\") |> Op.type?(\"insert\")\n    true\n\n    iex> Op.insert(\"Hello\") |> Op.type?(\"insert\", :string)\n    true\n\n    iex> Op.insert(\"Hello\") |> Op.type?(\"insert\", :number)\n    false\n\n    iex> Op.retain(1) |> Op.type?(\"retain\", :number)\n    true","ref":"Delta.Op.html#type?/3-examples"},{"type":"type","title":"Delta.Op.attributes/0","doc":"Stand-in type while attributes is keyed with String.t() instead of Atom.t()","ref":"Delta.Op.html#t:attributes/0"},{"type":"type","title":"Delta.Op.delete_op/0","doc":"Stand-in type while operators are keyed with String.t() instead of Atom.t()\n\n`%{delete: pos_integer()}`","ref":"Delta.Op.html#t:delete_op/0"},{"type":"type","title":"Delta.Op.insert_op/0","doc":"Stand-in type while operators are keyed with String.t() instead of Atom.t()\n\n`%{insert: String.t() | EmbedHandler.embed(), ...}`","ref":"Delta.Op.html#t:insert_op/0"},{"type":"type","title":"Delta.Op.operation/0","doc":"Stand-in type while operators are keyed with String.t() instead of Atom.t()","ref":"Delta.Op.html#t:operation/0"},{"type":"type","title":"Delta.Op.retain_op/0","doc":"Stand-in type while operators are keyed with String.t() instead of Atom.t()\n\n`%{retain: pos_integer() | EmbedHandler.embed()}`","ref":"Delta.Op.html#t:retain_op/0"},{"type":"type","title":"Delta.Op.t/0","doc":"","ref":"Delta.Op.html#t:t/0"},{"type":"extras","title":"Delta","doc":"# Delta\n\n[![Build Status][badge-github]][github-build]\n[![Version][badge-version]][hexpm]\n[![Downloads][badge-downloads]][hexpm]\n[![License][badge-license]][github-license]\n\n> Simple yet expressive format to describe documents' contents and changes 🗃\n\nDeltas are a simple, yet expressive format that can be used to describe contents and changes.\nThe format is a strict subset of JSON, is human readable, and easily parsible by machines.\nDeltas can describe any rich-text document, includes all text and formatting information,\nwithout the ambiguity and complexity of HTML.\n\nThe Delta format is suitable for [Operational Transform][wiki-ot] and can be used in real-time,\ncollaborative document editors (e.g. Slab, Google Docs). A walkthough of the motivation and\ndesign thinking behind Deltas are on [Designing the Delta Format][quill-delta].\n\nSee the [Documentation][docs].","ref":"readme.html"},{"type":"extras","title":"Installation - Delta","doc":"Add `delta` to your project dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:delta, \"~> 0.4.1\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Delta","doc":"A Delta is made up of a list of operations, which describe changes to a document. These can be\n`insert`, `delete` or `retain`. These operations do not take an index, but instead describe the\nchange at the current index. Retains are used to \"keep\" parts of the document.","ref":"readme.html#usage"},{"type":"extras","title":"Quick Example - Delta","doc":"```elixir\nalias Delta.Op\n\n# Document with text \"Gandalf the Grey\", with \"Gandalf\" bolded\n# and \"Grey\" in grey\ndelta = [\n  Op.insert(\"Gandalf\", %{\"bold\" => true}),\n  Op.insert(\" the \"),\n  Op.insert(\"Grey\", %{\"color\" => \"#ccc\"}),\n]\n\n# Define change intended to be applied to above:\n# Keep the first 12 characters, delete the next 4,\n# and insert a white \"White\"\ndeath = [\n  Op.retain(12),\n  Op.delete(4),\n  Op.insert(\"White\", %{\"color\" => \"#fff\"}),\n]\n\n\n# Applying the change:\nDelta.compose(delta, death)\n# => [\n#   %{\"insert\" => \"Gandalf\", \"attributes\" => %{\"bold\" => true}},\n#   %{\"insert\" => \" the \"},\n#   %{\"insert\" => \"White\", \"attributes\" => %{\"color\" => \"#fff\"}},\n# ]\n```","ref":"readme.html#quick-example"},{"type":"extras","title":"Operations - Delta","doc":"","ref":"readme.html#operations"},{"type":"extras","title":"Insert - Delta","doc":"Insert operations have an `insert` key defined. A String value represents inserting text. Any\nother type represents inserting an embed (however only one level of object comparison will be\nperformed for equality).\n\nIn both cases of text and embeds, an optional `attributes` key can be defined with a `map` to\ndescribe additonal formatting information. Formats can be changed by the `retain` operation.\n\n```elixir\n# Insert a text\nOp.insert(\"Some Text\")\n\n# Insert a bolded text\nOp.insert(\"Bolded Text\", %{\"bold\" => true})\n\n# Insert a link\nOp.insert(\"Google\", %{\"link\" => \"https://google.com\"})\n\n# Insert an embed\nOp.insert(%{\"image\" => \"https://app.com/logo.png\"}, %{\"alt\" => \"App Logo\"})\n\n# Insert another embed\nOp.insert(%{\"video\" => \"https://www.youtube.com/watch?v=dQw4w9WgXcQ\"}, %{\"width\" => 420, \"height\" => 315})\n```","ref":"readme.html#insert"},{"type":"extras","title":"Delete - Delta","doc":"Delete operations have a positive integer `delete` key defined representing the number of\ncharacters to delete. All embeds have a length of 1.\n\n```elixir\n# Delete the next 10 characters\nOp.delete(10)\n```","ref":"readme.html#delete"},{"type":"extras","title":"Retain - Delta","doc":"Retain operations have a positive integer `retain` key defined representing the number of\ncharacters to keep (other libraries might use the name keep or skip). An optional `attributes`\nkey can be defined with a `map` to describe formatting changes to the character range. A\nvalue of `nil` in the attributes map represents removal of that key.\n\nNote: It is not necessary to retain the last characters of a document as this is implied.\n\n```elixir\n# Keep the next 5 characters\nOp.retain(5)\n\n# Keep and bold the next 5 characters\nOp.retain(5, %{\"bold\" => true})\n\n# Keep and unbold the next 5 characters\nOp.retain(5, %{\"bold\" => nil})\n```","ref":"readme.html#retain"},{"type":"extras","title":"Operational Transform - Delta","doc":"Operational Transform (OT) is a technology for building collabortive experiences, and is\nespecially useful in application sharing and building real-time document editors that support\nmulti-user collaboration (e.g. Google Docs, Slab).\n\nDelta supports OT out of the box and can be very useful in employing Operational Transform\ntechniques in Elixir. It supports the following properties:","ref":"readme.html#operational-transform"},{"type":"extras","title":"Compose - Delta","doc":"Returns a new Delta that is equivalent to applying the operations of one Delta, followed\nby another Delta:\n\n```elixir\na = [Op.insert(\"abc\")]\nb = [Op.retain(1), Op.delete(1)]\n\nDelta.compose(a, b)\n# => [%{\"insert\" => \"ac\"}]\n```","ref":"readme.html#compose"},{"type":"extras","title":"Transform - Delta","doc":"Transforms given delta against another's operations. This accepts an optional `priority`\nargument (default: `false`), used to break ties. If `true`, the first delta takes priority\nover other, that is, its actions are considered to happen \"first.\"\n\n```elixir\na = [Op.insert(\"a\")]\nb = [Op.insert(\"b\"), Op.retain(5), Op.insert(\"c\")]\n\nDelta.transform(a, b, true)\n# => [\n#  %{\"retain\" => 1},\n#  %{\"insert\" => \"b\"},\n#  %{\"retain\" => 5},\n#  %{\"insert\" => \"c\"},\n# ]\n\nDelta.transform(a, b)\n# => [\n#  %{\"insert\" => \"b\"},\n#  %{\"retain\" => 6},\n#  %{\"insert\" => \"c\"},\n# ]\n```\n\nNote that even though `delete` operations support attributes, it is only safe to\nuse `transform` with deletes that don't have them.","ref":"readme.html#transform"},{"type":"extras","title":"Invert - Delta","doc":"Returns an inverted delta that has the opposite effect of against a base document delta.\nThat is `base |> Delta.compose(change) |> Delta.compose(inverted) == base`.\n\n```elixir\nbase = [Op.insert(\"Hello\\nWorld\")]\n\nchange = [\n  Op.retain(6, %{\"bold\" => true}),\n  Op.delete(5),\n  Op.insert(\"!\"),\n]\n\ninverted = Delta.invert(change, base)\n# => [\n#   %{\"retain\" => 6, \"attributes\" => %{\"bold\" => nil}},\n#   %{\"insert\" => \"World\"},\n#   %{\"delete\" => 1},\n# ]\n\nbase |> Delta.compose(change) |> Delta.compose(inverted) == base\n# => true\n```","ref":"readme.html#invert"},{"type":"extras","title":"Contributing - Delta","doc":"- [Fork][github-fork], Enhance, Send PR\n- Lock issues with any bugs or feature requests\n- Implement something from Roadmap\n- Spread the word :heart:\n\n \n\n[badge-github]: https://github.com/slab/delta-elixir/actions/workflows/ci.yml/badge.svg\n[badge-version]: https://img.shields.io/hexpm/v/delta.svg\n[badge-license]: https://img.shields.io/hexpm/l/delta.svg\n[badge-downloads]: https://img.shields.io/hexpm/dt/delta.svg\n[hexpm]: https://hex.pm/packages/delta\n[github-build]: https://github.com/slab/delta-elixir/actions/workflows/ci.yml\n[github-license]: https://github.com/slab/delta-elixir/blob/master/LICENSE\n[github-fork]: https://github.com/slab/delta-elixir/fork\n[docs]: https://hexdocs.pm/delta\n[wiki-ot]: https://en.wikipedia.org/wiki/Operational_transformation\n[quill-delta]: https://quilljs.com/guides/designing-the-delta-format/","ref":"readme.html#contributing"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.4.1 (2024-11-28) - Changelog","doc":"","ref":"changelog.html#v0-4-1-2024-11-28"},{"type":"extras","title":"Maintenances - Changelog","doc":"* Address Elixir 1.17 warnings","ref":"changelog.html#maintenances"},{"type":"extras","title":"v0.4.0 (2024-04-22) - Changelog","doc":"","ref":"changelog.html#v0-4-0-2024-04-22"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Delta.diff/2`\n  * Add `Delta.Attr.diff/2`\n  * Add `c:Delta.EmbedHandler.diff/2`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"* Fix `Delta.split/2` when index goes beyond end of delta #12\n  * Fix `Delta.split/2` when splitter returns 0 index #13","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.3.0 (2022-09-19) - Changelog","doc":"","ref":"changelog.html#v0-3-0-2022-09-19"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add support for attributes in delete operations","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.2.0 (2022-05-30) - Changelog","doc":"","ref":"changelog.html#v0-2-0-2022-05-30"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add `Delta.slice_max/3`","ref":"changelog.html#enhancements"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}